%==========================================================================
% demo_enron_subject.m:     example script for using the variational EM
%                           algorithm with the Enron data (by subject)
% Copyright: Jorge G. Silva, Duke University, 2007
%==========================================================================

%--------------------------------------------------------------------------
% Parameters and initialization
%--------------------------------------------------------------------------

clear all
close all
clc


dataset_name='enron';

data=load('enron_day','-ascii');
data=spconvert(data);
data=data>0; 

% Use every e-mail address (WARNING: that's p=75,511. Make sure you have enough memory.)
subset_col=1:size(data,2);
data=data(:,subset_col);

% Remove zero-valued columns and rows
hamm_row=sum(data,2);
hamm_col=sum(data,1);
nonzero_rows=find(hamm_row>1);
nonzero_cols=find(hamm_col>0);
data=data(nonzero_rows,nonzero_cols);
subset_rows=1:size(data,1);

% Use 800 points for training, the rest for testing
training_set=randperm(length(subset_rows));
training_set=training_set(1:800);
test_set=setdiff(subset_rows,training_set);

%--------------------------------------------------------------------------
% Use this line if you'd like to use the same training and test sets as in
% the paper. Otherwise, please comment it out.
load 'enron_day_0'...
    data nonzero_rows nonzero_cols subset_rows subset_col hamm_row hamm_col ...
    training_set test_set;
%--------------------------------------------------------------------------
X=data(training_set,:);
Z=data(test_set,:);

[nx,dummy]=size(X);
[nz,d]=size(Z);

% Parameters
m=1000;    % m= # MC samples
T=40;       % T= # of EM iterations

%--------------------------------------------------------------------------
% Run the variational EM algorithm
%--------------------------------------------------------------------------

[Fx,H,p,rho,r1x,r2x,L]=hyper_em(X,T);
Fz=hyperpmf_test(Z,H);

%--------------------------------------------------------------------------
% eta_i and gamma_i
%--------------------------------------------------------------------------

[r1z,r2z]=hyper_em_annotations(Fz,p,d);
eta_i=1-r1z(:,end);

[f0,F0,U0,xMC,fMC,alphaMC]=hyper_mc(X,m,'hyperpmf_test',H);
[f0z,F0z,U0z,zMC,fMC,alphaMC]=hyper_mc(Z,m,'hyperpmf_test',H);
gamma_i=p(end)*(U0z)./( (1-p(end))*(F0z)+p(end)*(U0z) );


%--------------------------------------------------------------------------
% Top 20 anomalies
%--------------------------------------------------------------------------

fprintf('--------------------------------------------------------------\n');
fprintf('Top anomalies sorted by f\n');
fprintf('--------------------------------------------------------------\n');
[dummy,top_anomalies]=sort(Fz(:,end),'ascend');
date_id=nonzero_rows(subset_rows(test_set(top_anomalies(1:20))));

% Read all dates
fid=fopen('email_date.mat');
aux=textscan(fid,'%d %s');

for i=1:20
    ind=find(aux{1}==date_id(i));
    date_txt{i}=aux{2}(ind);
end
fclose(fid);

% Output Top 20
fprintf('   | Obs. index | Days / Date\n');
fprintf('----------------------------------------------------------\n');    
for i=1:20
    fprintf('%2d | %10d | %4d   %s\n',i, top_anomalies(i), date_id(i), cell2mat(date_txt{i}));
end


%--------------------------------------------------------------------------
% Figures
%--------------------------------------------------------------------------

figure(1);clf
plot(H(1,:));
set(gca,'FontSize',16,'FontName','Times')
xlabel('vertex index $j$','interpreter','latex');
ylabel('$\hat{\theta}_j$','interpreter','latex');

%----------

figure(2);clf
subplot(311)
stem(1:nz,eta_i,'b','MarkerSize',2);
axis([0 nz -.1 1.5])
set(gca,'FontSize',16,'FontName','Times')
ylabel('$\eta_i$','interpreter','latex');


subplot(312)
stem(1:nz,gamma_i,'b','MarkerSize',2);
axis([0 nz -.1 1.5])
set(gca,'FontSize',16,'FontName','Times')
ylabel('$\gamma_i$','interpreter','latex');

subplot(313), plot(sum(Z,2),'k');
set(gca,'FontSize',16,'FontName','Times')
ylabel('$\|x_i\|$','interpreter','latex');
xlabel('observation index $i$','interpreter','latex');
aux=axis;
axis([0 nz aux(3) aux(4)])

%----------

figure(3);clf
subplot(211), hold on, plot(1:nz,Fz(:,end),'-b',1:nz,-d*log2(exp(1))*ones(1,nz),':b');
set(gca,'FontSize',16,'FontName','Times')
handle=legend('$\hat{f}(x_i)$','$\mu(x_i)$');
set(handle,'interpreter','latex','FontSize',16);
title('Test set');
aux=axis;
axis([0 nz aux(3) aux(4)])

subplot(212), plot(1:nz,sum(Z,2),'k')
set(gca,'FontSize',16,'FontName','Times')
ylabel('$\|x_i\|$','interpreter','latex');
xlabel('observation index $i$','interpreter','latex');
aux=axis;
axis([0 nz aux(3) aux(4)])

